#include <algorithm>
#include <vector>
#include <stdio.h>
#include <iostream>
#include <cassert>
#include <cstdint>
#include <cstring>
#include <ios>
#include <functional>
using namespace std;

#define ran(i, a, b) for (int i = (a); i < (int)(b); i++)
using ii = pair<int, int>;
#if defined(SHIROKO1_LOCAL) && !defined(STFU)
#define DEBUG(x) cerr << x << endl;
#else
#define DEBUG(...) ((void)0)
#endif

static void die(const char *what)
{
	perror(what);
	exit(1);
}

static void wrong_input(const char *what)
{
	fprintf(stderr, "invalid test case: %s\n", what);
	exit(1);
}

static void wrong_answer()
{
	fprintf(stdout, "0.0\n");
	fprintf(stderr, "translate:wrong\n");
	exit(0);
}

static void accepted()
{
	fprintf(stdout, "1.0\n");
	fprintf(stderr, "translate:success\n");
	exit(0);
}

static void check_tree(int n, const vector<ii> &edges)
{
	vector<vector<int>> nbors(n);
	vector<bool> seen(n, false);
	for (ii ab : edges) {
		nbors[ab.first].push_back(ab.second);
		nbors[ab.second].push_back(ab.first);
	}

	function<void(int, int)> dfs = [&](int a, int p) -> void {
		seen[a] = true;
		for (int b : nbors[a]) {
			if (b == p) {
				assert(seen[b]);
			} else {
				if (seen[b]) {
					wrong_input("input is not a tree");
				} else {
					dfs(b, a);
				}
			}
		}
	};
	dfs(0, 0);
	ran (i, 0, n) {
		if (! seen[i]) {
			wrong_input("input graph is not connected");
		}
	}
}

int main(int argc, char **argv)
{
	if (argc != 4) {
		fprintf(stderr, "usage: %s <input> <output> <contestant's output>\n", argv[0]);
		return 1;
	}
	const char *input_file = argv[1];
	const char *correct_output_file = argv[2];
	const char *output_file = argv[3];

	vector<int> isomorphism;
	vector<ii> edges[2];
	char correct_verdict[16];
	char verdict[16];

	FILE *f = fopen(input_file, "r");
	if (!f)
		die(input_file);
	int n;
	if (fscanf(f, "%i", &n) != 1)
		wrong_input("cannot read int");
	if (n < 1 || n > 200000)
		wrong_input("invalid input size");
	ran (j, 0, 2)
	ran (i, 0, n-1) {
		int a, b;
		if (fscanf(f, "%i%i", &a, &b) != 2)
			wrong_input("cannot read int");
		if (a == b)
			wrong_input("edge is a loop");
		if (a >= b)
			wrong_input("edge has wrong direction");
		if (a < 0 || a >= n)
			wrong_input("vertex label out of range");
		if (b < 0 || b >= n)
			wrong_input("vertex label out of range");
		if (a == b)
			wrong_input("tree contains a loop");
		edges[j].push_back({ a, b });
	}
	fclose(f);

	check_tree(n, edges[0]);
	check_tree(n, edges[1]);

	f = fopen(correct_output_file, "r");
	if (!f)
		die(correct_output_file);
	fscanf(f, "%16s", correct_verdict);
	fclose(f);

	f = fopen(output_file, "r");
	if (!f)
		die(output_file);
	fscanf(f, "%16s", verdict);
	if (strcmp(verdict, correct_verdict))
		wrong_answer();
	if (strcmp(verdict, "EI") == 0)
		accepted();
	ran (i, 0, n) {
		int a;
		if (fscanf(f, "%i", &a) != 1)
			wrong_answer();
		if (a < 0 || a >= n)
			wrong_answer();
		isomorphism.push_back(a);
	}
	fclose(f);

	vector<bool> seen(n, false);
	ran (i, 0, n) {
		if (seen[isomorphism[i]]) {
			DEBUG("WA: vertex " << i << " appears > 1 times");
			wrong_answer(); // not a bijection
		} else {
			seen[isomorphism[i]] = true;
		}
	}

	for (ii &e : edges[0]) {
		int a = isomorphism[e.first];
		int b = isomorphism[e.second];
		e = { min(a, b), max(a, b) };
	}
	for (ii &e : edges[1]) {
		int a = e.first;
		int b = e.second;
		e = { min(a, b), max(a, b) };
	}
	sort(edges[0].begin(), edges[0].end());
	sort(edges[1].begin(), edges[1].end());
	ran (i, 0, (int)edges[0].size()) {
		if (edges[0][i] != edges[1][i]) {
			DEBUG("WA: edge " << edges[1][i].first << "--" << edges[1][i].second << " not covered\n");
			wrong_answer(); // not an isomorphism
		}
	}
	accepted();
	return 0;
}
