#!/usr/bin/python3

import random as r
from math import *
import argparse
import sys
sys.setrecursionlimit(250000)

args = argparse.ArgumentParser()
args.add_argument('--size', default=10, type=float, help='max size of input')
args.add_argument('--damage', action='store_true', default=False, help='generate almost isomorphic trees')
args.add_argument('--independent', action='store_true', default=False, help='generate independant trees')
args.add_argument('--expand', type=int, default=1, help='turn edges into paths of this length')
subargs = args.add_subparsers(dest='type', help='test type')
sa = subargs.add_parser('random')
sa = subargs.add_parser('regular')
sa.add_argument('--degree', type=int, default=3, help='degree of the vertices')
sa = subargs.add_parser('star')
sa = subargs.add_parser('lsys')
sa.add_argument('--concat_portion', type=int, default=1, help='how often to concatternate super-edges')
sa.add_argument('--concat_factor', type=float, default=0.5, help='how many edges the concaternated super-edge has on average')
sa.add_argument('--same_join_portion', type=int, default=1, help='how often to join two equal super-edges')
sa.add_argument('--diff_join_portion', type=int, default=1, help='how often to join two different super-edges')
sa.add_argument('--diff_join_factor', type=float, default=0.5, help='how many edges the joined super-edge has on average')
args = args.parse_args()

def randomize(n, edges):
        p = list(range(n))
        r.shuffle(p)
        for i, (a, b) in enumerate(edges):
                edges[i] = p[a], p[b]
        r.shuffle(edges)
        return edges

def expand(n, ex, edges):
        i = n
        new_edges = []
        for a, b in edges:
                for _ in range(ex):
                        new_edges.append((a, i))
                        a = i
                        i += 1
                new_edges.append((a, b))
        return new_edges

def repair(n, edges):
        not_seen = set(range(n))
        nbors = [[] for _ in range(n)]
        for a, b in edges:
                nbors[a].append(b)
                nbors[b].append(a)
        comp = [0]
        not_seen.remove(0)
        i = 0
        while len(comp) < n:
                while i < len(comp):
                        a = comp[i]
                        i += 1
                        for b in nbors[a]:
                                if b in not_seen:
                                        comp.append(b)
                                        not_seen.remove(b)
                if len(comp) == n:
                        break
                b = r.choice(list(not_seen)) # XXX: potentially O(N**2)
                a = r.choice(comp)
                edges.append((a, b))
                comp.append(b)
                not_seen.remove(b)
        r.shuffle(edges)
        return edges

def pop_random(lst):
        i = r.randint(0, len(lst)-1)
        x = lst[i]
        lst[i] = lst[-1]
        lst.pop()
        return x

def renumber(edges, add):
    new_edges = []
    for a, b in edges:
        new_edges.append((a+add, b+add))
    return new_edges;

def relabel(edges, old_label, new_label):
    new_edges = []
    for a, b in edges:
        if a == old_label:
            a = new_label
        if b == old_label:
            b = new_label
        new_edges.append((a, b))
    return new_edges;

def L_generate(size): #generate super-edge from 0 to size
    if size == 1:
        return [(0,1)]
    total = args.concat_portion + args.same_join_portion + args.diff_join_portion
    type = r.randint(0, total-1)
    if size==2 or type < args.concat_portion:
        first_size = max(r.randint(0, min(int(size * args.concat_factor * 2), size-1)), 1)
        second_size = size - first_size
        return L_generate(first_size)+renumber(L_generate(second_size), first_size)
    elif type < args.concat_portion + args.same_join_portion:
        new_size = (size-1) // 2
        edges = L_generate(new_size)
        edges = edges + relabel(renumber(edges, new_size), new_size, 0)
        edges.append((2*new_size, 2*new_size+1))
        if new_size*2+1 < size:
            edges.append((2*new_size+1, size))
        return edges;
    else:
        first_size = max(r.randint(0, min(int(size * args.diff_join_factor * 2), size-1)), 1)
        second_size = size - first_size
        return L_generate(first_size) + relabel(renumber(L_generate(second_size), first_size), first_size, 0)

def generate(max_size):
        size = 0
        edges = []
        def new_vert():
                nonlocal size
                i = size
                size += 1
                return i

        if args.type == 'random':
                new_vert() # root
                for i in range(1, max_size):
                        edges.append((r.randint(0, i-1), new_vert()))
        elif args.type == 'regular':
                d = args.degree
                root = new_vert()
                q = []
                for i in range(d):
                        a = new_vert()
                        edges.append((root, a))
                        q.append(a)
                while size+d-1 <= max_size:
                        a = pop_random(q)
                        for _ in range(1, d):
                                b = new_vert()
                                q.append(b)
                                edges.append((a, b))
        elif args.type == 'star':
                root = new_vert()
                for _ in range(1, max_size):
                        edges.append((root, new_vert()))
        elif args.type == 'lsys':
            size = max_size
            edges = L_generate(size-1)
        else:
                raise ValueError('unknown generator type')
        return size, edges

max_size = r.randint(int(args.size*0.9), args.size-1)
if args.type == 'lsys' and args.damage:
    max_size = (max_size-3)//2
size, edges_a = generate(max_size)
if args.independent:
        while True:
                size_b, edges_b = generate(max_size)
                if size_b == size:
                        break
else:
        edges_b = edges_a[:]

if args.expand != 1:
        edges_a = expand(size, args.expand, edges_a)
        edges_b = expand(size, args.expand, edges_b)
        size += args.expand*(size-1)

edges_a = randomize(size, edges_a)
edges_b = randomize(size, edges_b)

if args.damage:
    if args.type == 'lsys':
        last_v = size-1
        edges_a = edges_a + renumber(edges_a, last_v+2)
        edges_a.append((0, last_v+1))
        edges_a.append((0, last_v+2))
        edges_b = edges_a[:]
        edges_a.append((last_v, 2*last_v+3))
        edges_b.append((2*last_v+2, 2*last_v+3))
        size = 2*last_v+4
    else:
        edges_b = edges_b[:-r.randint(1, max(1, size//100))]
        edges_b = repair(size, edges_b)

edges_a = randomize(size, edges_a)
edges_b = randomize(size, edges_b)

print(size)
for a, b in edges_a:
        print(min(a, b), max(a, b))
for a, b in edges_b:
        print(min(a, b), max(a, b))
