#!/bin/sh -e

time=$(which gtime || which time) # want GNU time, not BSD time
testcnt_file=$(mktemp)
trap 'rm $testcnt_file' EXIT
echo 0 > $testcnt_file

rm -f input/*.txt
rm -f output/*
rm -f gen/GEN

new_group() {
	echo "# ST: $1" >> gen/GEN
}

new_test() {
	test_i=$(cat $testcnt_file)
	echo "$test_i $@" >> gen/GEN
	echo input/input$test_i.txt
	expr $test_i + 1 > $testcnt_file
}

new_group 0
cp sample/input0.txt $(new_test example)
cp sample/input1.txt $(new_test example)

gen_test() {
	echo "generating $@..." >&2
	#name=$(echo "$*" | sed 's/[^a-z0-9]/_/g')
	local py
	py=python3
	if echo $@ | grep -q -e '--size 100000'; then
		py=pypy3
	fi
	$py ./gen/gen.py "$@" > $(new_test)
}

gen_all() {
	gen_test $@
	gen_test --damage $@
	gen_test --independent $@
}

gen_size() {
	n=$1
	gen_test --size $n regular --degree 3
	gen_test --damage --size $n regular --degree 3
	gen_test --size $n random
	gen_test --size $n star
  gen_test --size $n lsys --concat_portion 10000000 #line
}

new_group 10
gen_size 20
gen_all --size $n $@ random

new_group 20
gen_size 1000
gen_test --size 1000 lsys --concat_portion 10 --concat_factor 0.003 --diff_join_portion 10 --diff_join_factor 0.003 --same_join_portion 2 #dense
gen_all --size 1000 lsys --concat_portion 100 --concat_factor 0.003 --diff_join_portion 50 --diff_join_factor 0.001 --same_join_portion 1 #sparse

new_group 30
gen_size 100000
gen_test --size 100000 lsys --concat_portion 10 --concat_factor 0.001 --diff_join_portion 10 --diff_join_factor 0.001 --same_join_portion 1 #dense
gen_all --size 100000 lsys --concat_portion 300 --concat_factor 0.001 --diff_join_portion 150 --diff_join_factor 0.0005 --same_join_portion 1 #sparse
gen_test --size 100000 lsys --concat_portion 700 --concat_factor 0.001 --diff_join_portion 100 --diff_join_factor 0.0001 --same_join_portion 1 #extra-sparse

sed -i'' '/this line is autogenerated/d' task.yaml
echo "n_input: $(cat $testcnt_file) # this line is autogenerated" >> task.yaml

make -C solution
mkdir -p output
for input in $(ls input/*.txt | sort -V)
do
	output=output/output${input#input/input}
  cp $input isosis.txt
	$time -f "$input %U %M" ./solution/probabilistic
  cp isoval.txt $output
done
