#include <algorithm>
#include <unordered_set>
#include <vector>
#include <stdio.h>
#include <iostream>
#include <cassert>
#include <cstdint>
#include <ios>
#include <chrono>
using namespace std;

#define ran(i, a, b) for (int i = (a); i < (int)(b); i++)
using ll = long long;
using u64 = uint64_t;
using vi = vector<int>;
using ii = pair<int, int>;
#if defined(SHIROKO1_LOCAL) && !defined(STFU)
#define DEBUG(x) cerr << x << endl;
#else
#define DEBUG(...) ((void)0)
#endif

std::chrono::time_point<std::chrono::system_clock> start_time;
bool check_time(){
  auto cur_time = std::chrono::system_clock::now();
  std::chrono::duration<double> diff = cur_time-start_time;
  return diff.count() > 0.4;
}

static u64 mix(u64 a)
{
#if 0
	u64 mem[2] = { a, 0xdeadbeeffeebdaedull };
	asm volatile (
		"pxor %%xmm0, %%xmm0;"
		"movdqa (%0), %%xmm1;"
		"aesenc %%xmm0, %%xmm1;"
		"movdqa %%xmm1, (%0);"
		:
		: "r" (&mem[0])
		: "memory"
	);
	return mem[0];
#else
  /*
	a += 0xdeadbeeffeebdaedull;
	a ^= (a>>3)|(a<<64-3);
	a += (a>>9)|(a<<64-9);
	a ^= (a<<27)|(a>>64-27);
	a += (a<<17)|(a>>64-17);
  */	
  a^= a << 13;
	a^= a >> 7;
	a^= a << 17;
#endif
	return a;
}

struct Vert;

struct Edge{
	Vert *ends[2];
	u64 h=1;
	void hash();
};

struct Vert{
	vector<pair<Vert *, Edge *> > con;
	u64 h=1;
	void hash(){
		for(auto cur : con){
			h += cur.second->h;
		}
		h = mix(h);
	}
  int idx;
  void dfs(Vert *from, vector<pair<u64, int> > &res){
    res.emplace_back(h, idx);
    vector<pair<u64, Vert *> > nxt_ord;
    for(auto cur : con){
      if(cur.first != from){
        nxt_ord.emplace_back(cur.first->h, cur.first);
      }
    }
    sort(nxt_ord.begin(), nxt_ord.end());
    for(auto cur : nxt_ord){
      cur.second->dfs(this, res);
    }
  }
};

void Edge::hash(){
	for(int i=0; i<2; ++i){
		h += ends[i]->h;
	}
	h = mix(h);
}

struct Tree {
	int size;
	Edge *edges;
	Vert *nodes;
	Tree(int n) : size(n), edges(new Edge[size-1]), nodes(new Vert[size]) {
		ran (i, 0, size){
      nodes[i].idx = i;
    }
  }
	void read(FILE *f){
		ran (i, 0, size-1) {
			int a, b;
			fscanf(f, "%i%i", &a, &b);
			edges[i].ends[0] = nodes+a;
			edges[i].ends[1] = nodes+b;
			nodes[a].con.emplace_back(nodes+b, edges+i);
			nodes[b].con.emplace_back(nodes+a, edges+i);
		}
	}
	void init(){
		ran (i, 0, size)
			nodes[i].h += nodes[i].con.size();
	}
	void rehash(){
		ran (i, 0, size-1)
			edges[i].hash();
		ran (i, 0, size)
			nodes[i].hash();
	}
	vector<pair<u64, int> > output(){
    u64 maxh = 0;
    int start;
		ran (i, 0, size) {
      if(nodes[i].h > maxh){
        maxh = nodes[i].h;
        start = i;
      }
    }
		vector<pair<u64, int> > res;
    nodes[start].dfs(NULL, res);
		return res;
	}
};

static int *ans;

int main(){
  start_time = std::chrono::system_clock::now();
	FILE *in_file = fopen("isosis.txt", "rt");
	FILE *out_file = fopen("isoval.txt", "wt");
	int n;
	fscanf(in_file, "%i", &n);
  int max_iters = n;
	Tree A(n);
	Tree B(n);
	cerr << "reading tree..." << endl;
	A.read(in_file);
  A.init();
	cerr << "hashing tree..." << endl;
  int iters = 0;
	for(; !check_time() && iters<max_iters; ++iters){
    A.rehash();
  }
  cerr<<iters<<endl;
	cerr << "reading tree..." << endl;
	B.read(in_file);
  B.init();
	cerr << "hashing tree..." << endl;
	for(int i=0; i<iters; ++i){
    B.rehash();
  }
	cerr << "matching..." << endl;
	ans = new int[n];
	auto Aord = A.output();
	auto Bord = B.output();
	ran (i, 0, n) {
		if(Aord[i].first != Bord[i].first){
			fprintf(out_file, "EI\n");
			exit(0);
		}
		ans[Aord[i].second] = Bord[i].second;
	}
	cerr << "done" << endl;
	fprintf(out_file, "JAH\n");
	ran (i, 0, n)
		fprintf(out_file, "%i\n", ans[i]);
	return 0;
}
