#include <iostream>
#include <vector>
#include <string>
#include <map>
#include <deque>

using namespace std;

typedef pair<int, int> edge;

edge make_edge (int u, int v) {
  return make_pair(min(u, v), max(u, v));
}

namespace tree {
  const int MAX_V = 100005;
  vector<int> adj [MAX_V];
  string colors [3] = {string("punane"), string("sinine"), string("roheline")};
  map<edge, string> edge_color;

  void dfs (int vertex, int parent, int cur_color) {
    for (int nxt : adj[vertex]) {
      if (nxt != parent) {
        cur_color = (cur_color + 1) % 3;
        edge_color[make_edge(vertex, nxt)] = colors[cur_color];
        dfs(nxt, vertex, cur_color);
      }
    }
  }

  void solve (int vertexc, int edgec) {
    vector<edge> edges;
    for (int i = 0; i < edgec; i++) {
      int u, v;
      cin >> u >> v;

      adj[u].push_back(v);
      adj[v].push_back(u);
      edges.push_back(make_edge(u, v));
    }

    for (int i = 1; i <= vertexc; i++) {
      if ((int) adj[i].size() > 3) {
        cout << "EI SAA" << '\n';
        return;
      }
    }
    cout << "SAAB" << '\n';

    dfs(1, 0, 0);

    for (int i = 0; i < edgec; i++) {
      cout << edge_color[edges[i]] << '\n';
    }
  }
}

namespace graph {
  const int MAX_V = 100005;
  vector<int> adj [MAX_V];
  string colors [3] = {string("punane"), string("sinine"), string("roheline")};
  map<edge, string> edge_color;

  bool vis [MAX_V];
  deque<int> cur_cycle;
  bool find_cycle (int vertex, int parent) {
    cur_cycle.push_back(vertex);
    vis[vertex] = true;
    for (int nxt : adj[vertex]) {
      if (vis[nxt]) {
        if (nxt != parent) {
          while (cur_cycle.front() != nxt) {
            cur_cycle.pop_front();
          }
          return true;
        }
      } else if (find_cycle(nxt, vertex)) {
        return true;
      }
    }
    cur_cycle.pop_back();
    return false;
  }

  void paint_edges (int vertex, int cur_color) {
    vis[vertex] = true;

    for (int nxt : adj[vertex]) {
      if (!vis[nxt]) {
        edge_color[make_edge(vertex, nxt)] = colors[cur_color];
        paint_edges(nxt, (cur_color + 1) % 3);
      }
    }
  }        
  
  void solve (int vertexc, int edgec) {
    cout << "SAAB" << endl;
    vector<edge> edges;
    for (int i = 0; i < edgec; i++) {
      int u, v;
      cin >> u >> v;

      adj[u].push_back(v);
      adj[v].push_back(u);
      edges.push_back(make_edge(u, v));
    }

    find_cycle(1, 0);

    for (int i = 1; i <= vertexc; i++) {
      vis[i] = false;
    }

    for (int u : cur_cycle) {
      vis[u] = true;
    }

    /*
    for (int u : cur_cycle) {
      cerr << u << " ";
    }
    cerr << endl;
    */

    for (int i = 0; i < (int) cur_cycle.size() - 1; i++) {
      edge_color[make_edge(cur_cycle[i], cur_cycle[i + 1])] = colors[i % 2];
    }
    edge_color[make_edge(cur_cycle[0], cur_cycle.back())] = colors[2];

    paint_edges(cur_cycle[0], 1);
    paint_edges(cur_cycle.back(), ((int) cur_cycle.size() - 1) % 2);
    for (int i = 1; i < (int) cur_cycle.size() - 1; i++) {
      paint_edges(cur_cycle[i], 2);
    }

    for (edge e : edges) {
      if (edge_color.count(e) == 0) {
        cout << "sinine" << '\n';
      } else {
        cout << edge_color[e] << '\n';
      }
    }
  }
}

int main () {
  int vertexc, edgec;
  cin >> vertexc >> edgec;

  if (vertexc == edgec + 1) {
    tree::solve(vertexc, edgec);
  } else {
    graph::solve(vertexc, edgec);
  }
}
